﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace GettingStarted.Filter
{
    internal class FIRFilter
    {
        private double[] _coefficients;
        private double[] _buffer;
        private int _bufferIndex;
        private int _delay;

        public FIRFilter(double[] coefficients)
        {
            _coefficients = coefficients;
            _buffer = new double[coefficients.Length];
            _bufferIndex = 0;
            _delay = (_coefficients.Length - 1) / 2;
        }

        public double Process(double input)
        {
            _buffer[_bufferIndex] = input;
            double result = 0d;

            int index = _bufferIndex;
            for (int i = 0; i < _coefficients.Length; i++)
            {
                result += _coefficients[i] * _buffer[index];
                index = (index > 0) ? index - 1 : _buffer.Length - 1;
            }

            _bufferIndex = (_bufferIndex + 1) % _buffer.Length;

            return result;
        }

        public double[] ProcessSignal(double[] inputSignal)
        {
            int signalLength = inputSignal.Length;
            double[] outputSignal = new double[signalLength];

            for (int i = 0; i < signalLength; i++)
            {
                outputSignal[i] = Process(inputSignal[i]);
            }

            // 페이즈 지연 보상
            for (int i = 0; i < signalLength; i++)
            {
                if (i + _delay < signalLength)
                {
                    outputSignal[i] = outputSignal[i + _delay];
                }
                else
                {
                    outputSignal[i] = 0; // 지연 보상 후 남은 부분은 0으로 채움
                }
            }

            return outputSignal;
        }

        public static double[] DesignLowPassFilter(int length, double cutoffFrequnecy, double sampleRate)
        {
            double[] coefficients = new double[length];
            double omega_c = 2 * Math.PI * cutoffFrequnecy / sampleRate;
            int M = length - 1;

            for (int n = 0; n <= M; n++)
            {
                if (n - M / 2 == 0)
                {
                    coefficients[n] = omega_c / Math.PI;
                }
                else
                {
                    coefficients[n] = Math.Sin(omega_c * (n - M / 2)) / (Math.PI * (n - M / 2)) * (0.54 - 0.46 * Math.Cos(2 * Math.PI * n / M));
                }
            }

            return coefficients;
        }
    }
}
